package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletDualButtonActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get LED State")
    public @ActionOutput(name = "ledL", type="short")
           @ActionOutput(name = "ledR", type="short")
           Map<String, Object> brickletDualButtonGetLEDState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletDualButton.LEDState value = ((BrickletDualButton)this.handler.getDevice()).getLEDState();
        result.put("ledL", value.ledL);
        result.put("ledR", value.ledR);
        return result;
    }

    public static Map<String, Object> brickletDualButtonGetLEDState(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDualButtonActions) {
            return ((BrickletDualButtonActions) actions).brickletDualButtonGetLEDState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualButtonActions class.");
        }
    }

    @RuleAction(label = "Get Button State")
    public @ActionOutput(name = "buttonL", type="short")
           @ActionOutput(name = "buttonR", type="short")
           Map<String, Object> brickletDualButtonGetButtonState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletDualButton.ButtonState value = ((BrickletDualButton)this.handler.getDevice()).getButtonState();
        result.put("buttonL", value.buttonL);
        result.put("buttonR", value.buttonR);
        return result;
    }

    public static Map<String, Object> brickletDualButtonGetButtonState(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDualButtonActions) {
            return ((BrickletDualButtonActions) actions).brickletDualButtonGetButtonState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualButtonActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletDualButtonGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletDualButton.Identity value = ((BrickletDualButton)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletDualButtonGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDualButtonActions) {
            return ((BrickletDualButtonActions) actions).brickletDualButtonGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualButtonActions class.");
        }
    }
}
