package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIndustrialDualAnalogInV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickletIndustrialDualAnalogInV2GetVoltage(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialDualAnalogInV2)this.handler.getDevice()).getVoltage(channel);
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDualAnalogInV2GetVoltage(@Nullable ThingActions actions, int channel) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInV2Actions) {
            return ((BrickletIndustrialDualAnalogInV2Actions) actions).brickletIndustrialDualAnalogInV2GetVoltage(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInV2Actions class.");
        }
    }

    @RuleAction(label = "Get Channel LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletIndustrialDualAnalogInV2GetChannelLEDConfig(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialDualAnalogInV2)this.handler.getDevice()).getChannelLEDConfig(channel);
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDualAnalogInV2GetChannelLEDConfig(@Nullable ThingActions actions, int channel) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInV2Actions) {
            return ((BrickletIndustrialDualAnalogInV2Actions) actions).brickletIndustrialDualAnalogInV2GetChannelLEDConfig(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInV2Actions class.");
        }
    }

    @RuleAction(label = "Get Channel LED Status Config")
    public @ActionOutput(name = "min", type="int")
           @ActionOutput(name = "max", type="int")
           @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletIndustrialDualAnalogInV2GetChannelLEDStatusConfig(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialDualAnalogInV2.ChannelLEDStatusConfig value = ((BrickletIndustrialDualAnalogInV2)this.handler.getDevice()).getChannelLEDStatusConfig(channel);
        result.put("min", value.min);
        result.put("max", value.max);
        result.put("config", value.config);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDualAnalogInV2GetChannelLEDStatusConfig(@Nullable ThingActions actions, int channel) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInV2Actions) {
            return ((BrickletIndustrialDualAnalogInV2Actions) actions).brickletIndustrialDualAnalogInV2GetChannelLEDStatusConfig(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInV2Actions class.");
        }
    }

    @RuleAction(label = "Get Sample Rate")
    public @ActionOutput(name = "rate", type="int")
           Map<String, Object> brickletIndustrialDualAnalogInV2GetSampleRate(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialDualAnalogInV2)this.handler.getDevice()).getSampleRate();
        result.put("rate", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDualAnalogInV2GetSampleRate(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInV2Actions) {
            return ((BrickletIndustrialDualAnalogInV2Actions) actions).brickletIndustrialDualAnalogInV2GetSampleRate();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInV2Actions class.");
        }
    }

    @RuleAction(label = "Get Calibration")
    public @ActionOutput(name = "offset", type="int[]")
           @ActionOutput(name = "gain", type="int[]")
           Map<String, Object> brickletIndustrialDualAnalogInV2GetCalibration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialDualAnalogInV2.Calibration value = ((BrickletIndustrialDualAnalogInV2)this.handler.getDevice()).getCalibration();
        result.put("offset", value.offset);
        result.put("gain", value.gain);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDualAnalogInV2GetCalibration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInV2Actions) {
            return ((BrickletIndustrialDualAnalogInV2Actions) actions).brickletIndustrialDualAnalogInV2GetCalibration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInV2Actions class.");
        }
    }

    @RuleAction(label = "Get ADC Values")
    public @ActionOutput(name = "value", type="int[]")
           Map<String, Object> brickletIndustrialDualAnalogInV2GetADCValues(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int[] value = ((BrickletIndustrialDualAnalogInV2)this.handler.getDevice()).getADCValues();
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDualAnalogInV2GetADCValues(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInV2Actions) {
            return ((BrickletIndustrialDualAnalogInV2Actions) actions).brickletIndustrialDualAnalogInV2GetADCValues();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletIndustrialDualAnalogInV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletIndustrialDualAnalogInV2)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDualAnalogInV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInV2Actions) {
            return ((BrickletIndustrialDualAnalogInV2Actions) actions).brickletIndustrialDualAnalogInV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletIndustrialDualAnalogInV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialDualAnalogInV2)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDualAnalogInV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInV2Actions) {
            return ((BrickletIndustrialDualAnalogInV2Actions) actions).brickletIndustrialDualAnalogInV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletIndustrialDualAnalogInV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialDualAnalogInV2)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDualAnalogInV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInV2Actions) {
            return ((BrickletIndustrialDualAnalogInV2Actions) actions).brickletIndustrialDualAnalogInV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletIndustrialDualAnalogInV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialDualAnalogInV2)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDualAnalogInV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInV2Actions) {
            return ((BrickletIndustrialDualAnalogInV2Actions) actions).brickletIndustrialDualAnalogInV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletIndustrialDualAnalogInV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialDualAnalogInV2.SPITFPErrorCount value = ((BrickletIndustrialDualAnalogInV2)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDualAnalogInV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInV2Actions) {
            return ((BrickletIndustrialDualAnalogInV2Actions) actions).brickletIndustrialDualAnalogInV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletIndustrialDualAnalogInV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialDualAnalogInV2.Identity value = ((BrickletIndustrialDualAnalogInV2)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDualAnalogInV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInV2Actions) {
            return ((BrickletIndustrialDualAnalogInV2Actions) actions).brickletIndustrialDualAnalogInV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInV2Actions class.");
        }
    }
}
