package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletPTCActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletPTCGetTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletPTC)this.handler.getDevice()).getTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletPTCGetTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPTCActions) {
            return ((BrickletPTCActions) actions).brickletPTCGetTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCActions class.");
        }
    }

    @RuleAction(label = "Is Sensor Connected")
    public @ActionOutput(name = "connected", type="boolean")
           Map<String, Object> brickletPTCIsSensorConnected(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickletPTC)this.handler.getDevice()).isSensorConnected();
        result.put("connected", value);
        return result;
    }

    public static Map<String, Object> brickletPTCIsSensorConnected(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPTCActions) {
            return ((BrickletPTCActions) actions).brickletPTCIsSensorConnected();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCActions class.");
        }
    }

    @RuleAction(label = "Get Wire Mode")
    public @ActionOutput(name = "mode", type="short")
           Map<String, Object> brickletPTCGetWireMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickletPTC)this.handler.getDevice()).getWireMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletPTCGetWireMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPTCActions) {
            return ((BrickletPTCActions) actions).brickletPTCGetWireMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCActions class.");
        }
    }

    @RuleAction(label = "Get Resistance")
    public @ActionOutput(name = "resistance", type="int")
           Map<String, Object> brickletPTCGetResistance(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletPTC)this.handler.getDevice()).getResistance();
        result.put("resistance", value);
        return result;
    }

    public static Map<String, Object> brickletPTCGetResistance(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPTCActions) {
            return ((BrickletPTCActions) actions).brickletPTCGetResistance();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCActions class.");
        }
    }

    @RuleAction(label = "Get Noise Rejection Filter")
    public @ActionOutput(name = "filter", type="short")
           Map<String, Object> brickletPTCGetNoiseRejectionFilter(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickletPTC)this.handler.getDevice()).getNoiseRejectionFilter();
        result.put("filter", value);
        return result;
    }

    public static Map<String, Object> brickletPTCGetNoiseRejectionFilter(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPTCActions) {
            return ((BrickletPTCActions) actions).brickletPTCGetNoiseRejectionFilter();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletPTCGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletPTC.Identity value = ((BrickletPTC)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletPTCGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPTCActions) {
            return ((BrickletPTCActions) actions).brickletPTCGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCActions class.");
        }
    }
}
